from qgl import render
from qgl import scene
from qgl import texture

def demo(test_image_filename, test_font_filename):
    
    import pygame
    import qgl


    def main():
        pygame.init()
        flags =  pygame.OPENGL|pygame.DOUBLEBUF|pygame.HWSURFACE
        pygame.display.set_mode((320,240), flags)
        
        compiler = qgl.render.Compiler()
        render = qgl.render.Render()
        picker = qgl.render.Picker()
        clock = pygame.time.Clock()
        
        ##build a scene graph structure like this
        #root
            #viewport
                #transform
                    #group (text)
            #viewport
                #transform
                    #group (texture, quad)
             
        
        root_node = qgl.scene.Root()
        v1 = qgl.scene.Viewport()
        v2 = qgl.scene.Viewport()
        t1 = qgl.scene.Transform()
        t2 = qgl.scene.Transform()
        g1 = qgl.scene.Group()
        g2 = qgl.scene.Group()
        
        g1.add(qgl.scene.Text('testing 1 2 3', test_font_filename))
        g1.selectable = True
        g2.add(qgl.scene.Texture(test_image_filename))
        g2.add(qgl.scene.Quad((5,5)))
        g2.selectable = True
        
        t1.translate = (-5.0, 0.0, -15.0)
        t2.translate = (5.0, 0.0, -15.0)
        
        v1.screen_dimensions = (0,0,320,120)
        v1.aspect = 320.0 / 120.0
        v2.screen_dimensions = (0,120,320,120)
        v2.aspect = 320.0 / 120.0

        root_node.add(v1,v2)
        v1.add(t1)
        v2.add(t2)
        t1.add(g1)
        t2.add(g2)
        
        #compile the scene graph
        root_node.accept(compiler)
            
        while True:
            for event in pygame.event.get():
                if event.type is pygame.QUIT:
                    return
                elif event.type is pygame.MOUSEBUTTONDOWN:
                    #set position we are interested in
                    picker.set_position(event.pos)
                    root_node.accept(picker)
                    for hit in picker.hits:
                        hit.parent.angle += 10
            
            #render to screen
            root_node.accept(render)
            clock.tick(60)
            pygame.display.flip()
            
            
    main()